#include "CSpielfeld.h"

/********************************************************************************************\
** CSpielfeld - Funktionen zum Steuern des Spielfeldes****************************************
\********************************************************************************************/
CSpielfeld::CSpielfeld(short screen_breite, short screen_hoehe, short feld_breite, short feld_hoehe)
{	
	// Fenster erzeugen
	screen = SDL_SetVideoMode(screen_breite,screen_hoehe,8,SDL_HWSURFACE|SDL_DOUBLEBUF);
	if(!screen)
	{
		fprintf(stderr,"Konnte Bildschirmmodus nicht setzten: %s\n",SDL_GetError());
		exit(1);
	}

	// Surface des Spielfeldes erstellen
	SDL_Surface* tmp=SDL_CreateRGBSurface(SDL_SWSURFACE, feld_breite, feld_hoehe, 8, 0, 0, 0, 0);
	sfeld=SDL_DisplayFormat(tmp);
	SDL_FreeSurface(tmp);

	// Surface des gezoomten Spielfeldes erstellen
	tmp=SDL_CreateRGBSurface(SDL_SWSURFACE, screen_breite, screen_hoehe, 8, 0, 0, 0, 0);
	sfeld_zoom=SDL_DisplayFormat(tmp);
	SDL_FreeSurface(tmp);

	// gre bernehmen
	breite=feld_breite;
	hoehe=feld_hoehe;

	// mein_feld erzeugen
	mein_feld=new bool*[breite];
	for(int i=0;i<breite;i++)
		mein_feld[i]=new bool[hoehe];

	// Farbwehrte laden
	farben.hint			= SDL_MapRGB(sfeld->format,255,255,255);
	farben.player_old	= SDL_MapRGB(sfeld->format,  0,128,  0);
	farben.player_new	= SDL_MapRGB(sfeld->format,  0,255,  0); 
	farben.gegner		= SDL_MapRGB(sfeld->format,  0,  0,255);

	// felder leeren
	clear();

	// screen mit Hintergrundfarbe fllen
	SDL_FillRect(screen,0,0xff);

	// Zeichnen
	zeichnen();
}

CSpielfeld::~CSpielfeld()
{
	if(sfeld) SDL_FreeSurface(sfeld);
	if(sfeld_zoom) SDL_FreeSurface(sfeld_zoom);

	if(mein_feld)
	{
		for(int i=0;i<breite;i++)
			delete [] mein_feld[i];
		delete [] mein_feld;
	}
}

void CSpielfeld::zeichnen()
{
	// sfeld zoomen
    zoomSurfaceY (sfeld, sfeld_zoom);

	// auf screen blitten
	SDL_BlitSurface(sfeld_zoom,0,screen,0);

	// auf Bildschirm bertragen
	SDL_Flip(screen);
}

Uint32 CSpielfeld::getpixel(int x, int y)
{
    int bpp = sfeld->format->BytesPerPixel;
    Uint8 *p = (Uint8 *)sfeld->pixels + y * sfeld->pitch + x * bpp;

    switch(bpp) {
    case 1:
        return *p;

    case 2:
        return *(Uint16 *)p;

    case 3:
        if(SDL_BYTEORDER == SDL_BIG_ENDIAN)
            return p[0] << 16 | p[1] << 8 | p[2];
        else
            return p[0] | p[1] << 8 | p[2] << 16;

    case 4:
        return *(Uint32 *)p;

    default:
        return 0;
    }
}

void CSpielfeld::putpixel(int x, int y, Uint32 pixel)
{
    int bpp = sfeld->format->BytesPerPixel;
    Uint8 *p = (Uint8 *)sfeld->pixels + y * sfeld->pitch + x * bpp;

    switch(bpp) {
    case 1:
        *p = pixel;
        break;

    case 2:
        *(Uint16 *)p = pixel;
        break;

    case 3:
        if(SDL_BYTEORDER == SDL_BIG_ENDIAN) {
            p[0] = (pixel >> 16) & 0xff;
            p[1] = (pixel >> 8) & 0xff;
            p[2] = pixel & 0xff;
        } else {
            p[0] = pixel & 0xff;
            p[1] = (pixel >> 8) & 0xff;
            p[2] = (pixel >> 16) & 0xff;
        }
        break;

    case 4:
        *(Uint32 *)p = pixel;
        break;
    }
}

void CSpielfeld::clear()
{
	int i,j;

	// Feld mit komplett mir hintergrundfarbe fllen
	SDL_FillRect(sfeld,0,farben.hint);

	// Rand des Spielfeldes mit der Farbe des Spielers fllen
	lock();
	for (i=0;i<breite;i++)
	{
		putpixel(i,0,farben.player_old);
		putpixel(i,hoehe-1,farben.player_old);
		mein_feld[i][0]=true;
		mein_feld[i][hoehe-1]=true;
	}
	for (i=1;i<hoehe;i++)
	{
		putpixel(0,i,farben.player_old);
		putpixel(breite-1,i,farben.player_old);
		mein_feld[0][i]=true;
		mein_feld[breite-1][i]=true;
	}
	unlock();

	// rest von mein_feld mit false fllen
	for(i=1;i<breite-1;i++)
		for(j=1;j<hoehe-1;j++)
			mein_feld[i][j]=false;

	// player_action zurcksetzen;
	player_action=false;
}



int CSpielfeld::zoomSurfaceY (SDL_Surface * src, SDL_Surface * dst)
{
  Uint32 x, y, sx, sy, *sax, *say, *csax, *csay, csx, csy;
  Uint8 *sp, *dp, *csp;
  int dgap;

  /* Variable setup */
  sx = (Uint32) (65536.0 * (float) src->w / (float) dst->w);
  sy = (Uint32) (65536.0 * (float) src->h / (float) dst->h);

  /* Allocate memory for row increments */
  if ((sax = (Uint32 *) malloc (dst->w * sizeof (Uint32))) == NULL)
    {
      return (-1);
    }
  if ((say = (Uint32 *) malloc (dst->h * sizeof (Uint32))) == NULL)
    {
      if (sax != NULL)
	{
	  free (sax);
	}
      return (-1);
    }

  /* Precalculate row increments */
  csx = 0;
  csax = sax;
  for (x = 0; x < dst->w; x++)
    {
      csx += sx;
      *csax = (csx >> 16);
      csx &= 0xffff;
      csax++;
    }
  csy = 0;
  csay = say;
  for (y = 0; y < dst->h; y++)
    {
      csy += sy;
      *csay = (csy >> 16);
      csy &= 0xffff;
      csay++;
    }

  csx = 0;
  csax = sax;
  for (x = 0; x < dst->w; x++)
    {
      csx += (*csax);
      csax++;
    }
  csy = 0;
  csay = say;
  for (y = 0; y < dst->h; y++)
    {
      csy += (*csay);
      csay++;
    }

  /* Pointer setup */
  sp = csp = (Uint8 *) src->pixels;
  dp = (Uint8 *) dst->pixels;
  dgap = dst->pitch - dst->w;

  /* Draw */
  csay = say;
  for (y = 0; y < dst->h; y++)
    {
      csax = sax;
      sp = csp;
      for (x = 0; x < dst->w; x++)
	{
	  /* Draw */
	  *dp = *sp;
	  /* Advance source pointers */
	  sp += (*csax);
	  csax++;
	  /* Advance destination pointer */
	  dp++;
	}
      /* Advance source pointer (for row) */
      csp += ((*csay) * src->pitch);
      csay++;
      /* Advance destination pointers */
      dp += dgap;
    }

  /* Remove temp arrays */
  free (sax);
  free (say);

  return (0);
}
